@@ -51,6 +51,9 @@ enum aws_http_errors {
     AWS_ERROR_HTTP_PROXY_CONNECT_FAILED_RETRYABLE,
     AWS_ERROR_HTTP_PROTOCOL_SWITCH_FAILURE,
     AWS_ERROR_HTTP_MAX_CONCURRENT_STREAMS_EXCEEDED,
+    AWS_ERROR_HTTP_STREAM_MANAGER_SHUTTING_DOWN,
+    AWS_ERROR_HTTP_STREAM_MANAGER_CONNECTION_ACQUIRE_FAILURE,
+    AWS_ERROR_HTTP_STREAM_MANAGER_UNEXPECTED_HTTP_VERSION,
 
     AWS_ERROR_HTTP_END_RANGE = AWS_ERROR_ENUM_END_RANGE(AWS_C_HTTP_PACKAGE_ID)
 };
@@ -82,6 +85,7 @@ enum aws_http_log_subject {
     AWS_LS_HTTP_SERVER,
     AWS_LS_HTTP_STREAM,
     AWS_LS_HTTP_CONNECTION_MANAGER,
+    AWS_LS_HTTP_STREAM_MANAGER,
     AWS_LS_HTTP_WEBSOCKET,
     AWS_LS_HTTP_WEBSOCKET_SETUP,
     AWS_LS_HTTP_PROXY_NEGOTIATION,
