@@ -34,7 +34,7 @@
         }                                                                                                              \
     } while (false)
 
-#if DEBUG_BUILD
+#if ASSERT_LOCK_HELD
 #    define ASSERT_SYNCED_DATA_LOCK_HELD(object)                                                                       \
         {                                                                                                              \
             int cached_error = aws_last_error();                                                                       \
@@ -156,10 +156,11 @@ struct aws_mqtt_client_connection {
         struct aws_byte_buf payload;
     } will;
     struct {
-        uint64_t current;      /* seconds */
-        uint64_t min;          /* seconds */
-        uint64_t max;          /* seconds */
-        uint64_t next_attempt; /* milliseconds */
+        uint64_t current_sec;                 /* seconds */
+        uint64_t min_sec;                     /* seconds */
+        uint64_t max_sec;                     /* seconds */
+        uint64_t next_attempt_ms;             /* milliseconds */
+        uint64_t next_attempt_reset_timer_ns; /* nanoseconds */
     } reconnect_timeouts;
 
     /* User connection callbacks */
