@@ -72,28 +72,16 @@ void aws_s3_meta_request_unlock_synced_data(struct aws_s3_meta_request *meta_req
     aws_mutex_unlock(&meta_request->synced_data.lock);
 }
 
-static bool s_is_get_request(const struct aws_s3_meta_request_options *options) {
-    if (options->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
-        return true;
-    }
-    if (options->type == AWS_S3_META_REQUEST_TYPE_DEFAULT) {
-        struct aws_byte_cursor method;
-        int err = aws_http_message_get_request_method(options->message, &method);
-        AWS_ASSERT(err == AWS_OP_SUCCESS);
-        (void)err;
-        return aws_byte_cursor_eq(&method, &aws_http_method_get);
-    }
-    return false;
-}
-
 static int s_meta_request_get_response_headers_checksum_callback(
-    /* TODO take in a priority list of checksum algorithms, and use this list to determine which checksum algorithm to
-       validate */
     struct aws_s3_meta_request *meta_request,
     const struct aws_http_headers *headers,
     int response_status,
     void *user_data) {
-    for (int i = AWS_SCA_INIT; i < AWS_SCA_COUNT; i++) {
+    for (int i = AWS_SCA_INIT; i <= AWS_SCA_END; i++) {
+        if (!aws_s3_meta_request_checksum_config_has_algorithm(meta_request, i)) {
+            /* If user doesn't select this algorithm, skip */
+            continue;
+        }
         const struct aws_byte_cursor *algorithm_header_name = aws_get_http_header_name_from_algorithm(i);
         if (aws_http_headers_has(headers, *algorithm_header_name)) {
             struct aws_byte_cursor header_sum;
@@ -101,6 +89,7 @@ static int s_meta_request_get_response_headers_checksum_callback(
             size_t encoded_len = 0;
             aws_base64_compute_encoded_len(aws_get_digest_size_from_algorithm(i), &encoded_len);
             if (header_sum.len == encoded_len - 1) {
+                /* encoded_len includes the nullptr length. -1 is the expected length. */
                 aws_byte_buf_init_copy_from_cursor(
                     &meta_request->meta_request_level_response_header_checksum, aws_default_allocator(), header_sum);
                 meta_request->meta_request_level_running_response_sum = aws_checksum_new(aws_default_allocator(), i);
@@ -124,6 +113,7 @@ static int s_meta_request_get_response_body_checksum_callback(
     if (meta_request->meta_request_level_running_response_sum) {
         aws_checksum_update(meta_request->meta_request_level_running_response_sum, body);
     }
+
     if (meta_request->body_user_callback_after_checksum) {
         return meta_request->body_user_callback_after_checksum(meta_request, body, range_start, user_data);
     } else {
@@ -161,9 +151,6 @@ static void s_meta_request_get_response_finish_checksum_callback(
         if (!aws_byte_buf_eq(&encoded_response_body_sum, &meta_request->meta_request_level_response_header_checksum)) {
             mut_meta_request_result->error_code = AWS_ERROR_S3_RESPONSE_CHECKSUM_MISMATCH;
         }
-    } else {
-        mut_meta_request_result->did_validate = false;
-        mut_meta_request_result->validation_algorithm = AWS_SCA_NONE;
     }
     if (meta_request->finish_user_callback_after_checksum) {
         meta_request->finish_user_callback_after_checksum(meta_request, meta_request_result, user_data);
@@ -179,8 +166,6 @@ int aws_s3_meta_request_init_base(
     struct aws_s3_client *client,
     size_t part_size,
     bool should_compute_content_md5,
-    const enum aws_s3_checksum_algorithm checksum_algorithm,
-    bool validate_get_response_checksum,
     const struct aws_s3_meta_request_options *options,
     void *impl,
     struct aws_s3_meta_request_vtable *vtable,
@@ -212,9 +197,7 @@ int aws_s3_meta_request_init_base(
 
     *((size_t *)&meta_request->part_size) = part_size;
     *((bool *)&meta_request->should_compute_content_md5) = should_compute_content_md5;
-    *((enum aws_s3_checksum_algorithm *)&meta_request->checksum_algorithm) = checksum_algorithm;
-    *((bool *)&meta_request->validate_get_response_checksum) = validate_get_response_checksum;
-
+    checksum_config_init(&meta_request->checksum_config, options->checksum_config);
     if (options->signing_config) {
         meta_request->cached_signing_config = aws_cached_signing_config_new(allocator, options->signing_config);
     }
@@ -241,6 +224,7 @@ int aws_s3_meta_request_init_base(
         aws_s3_client_acquire(client);
         meta_request->client = client;
         meta_request->io_event_loop = aws_event_loop_group_get_next_loop(client->body_streaming_elg);
+        meta_request->synced_data.read_window_running_total = client->initial_read_window;
     }
 
     meta_request->synced_data.next_streaming_part = 1;
@@ -250,7 +234,8 @@ int aws_s3_meta_request_init_base(
     meta_request->shutdown_callback = options->shutdown_callback;
     meta_request->progress_callback = options->progress_callback;
 
-    if (s_is_get_request(options)) {
+    if (meta_request->checksum_config.validate_response_checksum) {
+        /* TODO: the validate for auto range get should happen for each response received. */
         meta_request->headers_user_callback_after_checksum = options->headers_callback;
         meta_request->body_user_callback_after_checksum = options->body_callback;
         meta_request->finish_user_callback_after_checksum = options->finish_callback;
@@ -267,6 +252,37 @@ int aws_s3_meta_request_init_base(
     return AWS_OP_SUCCESS;
 }
 
+void aws_s3_meta_request_increment_read_window(struct aws_s3_meta_request *meta_request, uint64_t bytes) {
+    AWS_PRECONDITION(meta_request);
+
+    if (bytes == 0) {
+        return;
+    }
+
+    if (!meta_request->client->enable_read_backpressure) {
+        AWS_LOGF_DEBUG(
+            AWS_LS_S3_META_REQUEST,
+            "id=%p: Ignoring call to increment read window. This client has not enabled read backpressure.",
+            (void *)meta_request);
+        return;
+    }
+
+    AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "id=%p: Incrementing read window by %" PRIu64, (void *)meta_request, bytes);
+
+    /* BEGIN CRITICAL SECTION */
+    aws_s3_meta_request_lock_synced_data(meta_request);
+
+    /* Response will never approach UINT64_MAX, so do a saturating sum instead of worrying about overflow */
+    meta_request->synced_data.read_window_running_total =
+        aws_add_u64_saturating(bytes, meta_request->synced_data.read_window_running_total);
+
+    aws_s3_meta_request_unlock_synced_data(meta_request);
+    /* END CRITICAL SECTION */
+
+    /* Schedule the work task, to continue processing the meta-request */
+    aws_s3_client_schedule_process_work(meta_request->client);
+}
+
 void aws_s3_meta_request_cancel(struct aws_s3_meta_request *meta_request) {
     /* BEGIN CRITICAL SECTION */
     aws_s3_meta_request_lock_synced_data(meta_request);
@@ -383,8 +399,8 @@ static void s_s3_meta_request_destroy(void *user_data) {
 
     aws_cached_signing_config_destroy(meta_request->cached_signing_config);
     aws_mutex_clean_up(&meta_request->synced_data.lock);
-    /* For a meta request created by client, the endpoint should always be cleaned up by meta request finish call. Just
-     * use regular release here for those meta request not really went through the client */
+    /* endpoint should have already been released and set NULL by the meta request finish call.
+     * But call release() again, just in case we're tearing down a half-initialized meta request */
     aws_s3_endpoint_release(meta_request->endpoint);
     aws_s3_client_release(meta_request->client);
 
@@ -618,20 +634,11 @@ static void s_s3_meta_request_sign_request(
     void *user_data) {
     AWS_PRECONDITION(meta_request);
     AWS_PRECONDITION(meta_request->vtable);
-    AWS_PRECONDITION(meta_request->vtable->send_request_finish);
+    AWS_PRECONDITION(meta_request->vtable->sign_request);
 
     meta_request->vtable->sign_request(meta_request, request, on_signing_complete, user_data);
 }
 
-static bool s_is_put_request(const struct aws_s3_meta_request *meta_request) {
-    struct aws_byte_cursor method;
-    if (meta_request->type == AWS_S3_META_REQUEST_TYPE_DEFAULT &&
-        !aws_http_message_get_request_method(meta_request->initial_request_message, &method)) {
-        return aws_byte_cursor_eq(&method, &aws_http_method_put);
-    }
-    return meta_request->type == AWS_S3_META_REQUEST_TYPE_PUT_OBJECT;
-}
-
 /* Handles signing a message for the caller. */
 void aws_s3_meta_request_sign_request_default(
     struct aws_s3_meta_request *meta_request,
@@ -685,9 +692,9 @@ void aws_s3_meta_request_sign_request_default(
         return;
     }
 
-    /* if a checksum algorithm is set for a put request the request will be chunked and we need to change the signed
-     * body value to match this type of request*/
-    if (meta_request->checksum_algorithm != AWS_SCA_NONE && s_is_put_request(meta_request) &&
+    /* If the checksum is configured to be added to the trailer, the payload will be aws-chunked encoded. The payload
+     * will need to be streaming signed/unsigned. */
+    if (meta_request->checksum_config.location == AWS_SCL_TRAILER &&
         aws_byte_cursor_eq(&signing_config.signed_body_value, &g_aws_signed_body_value_unsigned_payload)) {
         signing_config.signed_body_value = g_aws_signed_body_value_streaming_unsigned_payload_trailer;
     }
@@ -851,9 +858,14 @@ static bool s_header_value_from_list(
 
 static void s_get_part_response_headers_checksum_helper(
     struct aws_s3_connection *connection,
+    struct aws_s3_meta_request *meta_request,
     const struct aws_http_header *headers,
     size_t headers_count) {
-    for (int i = AWS_SCA_INIT; i < AWS_SCA_COUNT; i++) {
+    for (int i = AWS_SCA_INIT; i <= AWS_SCA_END; i++) {
+        if (!aws_s3_meta_request_checksum_config_has_algorithm(meta_request, i)) {
+            /* If user doesn't select this algorithm, skip */
+            continue;
+        }
         const struct aws_byte_cursor *algorithm_header_name = aws_get_http_header_name_from_algorithm(i);
         struct aws_byte_cursor header_sum;
         if (s_header_value_from_list(headers, headers_count, algorithm_header_name, &header_sum)) {
@@ -883,28 +895,29 @@ static void s_get_response_part_finish_checksum_helper(struct aws_s3_connection
     struct aws_byte_buf encoded_response_body_sum;
     AWS_ZERO_STRUCT(response_body_sum);
     AWS_ZERO_STRUCT(encoded_response_body_sum);
-    if (error_code == AWS_OP_SUCCESS && connection->request->request_level_running_response_sum) {
+
+    struct aws_s3_request *request = connection->request;
+    if (error_code == AWS_OP_SUCCESS && request->request_level_running_response_sum) {
         size_t encoded_checksum_len = 0;
-        connection->request->did_validate = true;
-        aws_base64_compute_encoded_len(
-            connection->request->request_level_running_response_sum->digest_size, &encoded_checksum_len);
+        request->did_validate = true;
+        aws_base64_compute_encoded_len(request->request_level_running_response_sum->digest_size, &encoded_checksum_len);
         aws_byte_buf_init(&encoded_response_body_sum, aws_default_allocator(), encoded_checksum_len);
         aws_byte_buf_init(
-            &response_body_sum,
-            aws_default_allocator(),
-            connection->request->request_level_running_response_sum->digest_size);
-        aws_checksum_finalize(connection->request->request_level_running_response_sum, &response_body_sum, 0);
+            &response_body_sum, aws_default_allocator(), request->request_level_running_response_sum->digest_size);
+        aws_checksum_finalize(request->request_level_running_response_sum, &response_body_sum, 0);
         struct aws_byte_cursor response_body_sum_cursor = aws_byte_cursor_from_buf(&response_body_sum);
         aws_base64_encode(&response_body_sum_cursor, &encoded_response_body_sum);
-        connection->request->checksum_match =
-            aws_byte_buf_eq(&encoded_response_body_sum, &connection->request->request_level_response_header_checksum);
+        request->checksum_match =
+            aws_byte_buf_eq(&encoded_response_body_sum, &request->request_level_response_header_checksum);
+
+        request->validation_algorithm = request->request_level_running_response_sum->algorithm;
         aws_byte_buf_clean_up(&response_body_sum);
         aws_byte_buf_clean_up(&encoded_response_body_sum);
-        aws_checksum_destroy(connection->request->request_level_running_response_sum);
-        aws_byte_buf_clean_up(&connection->request->request_level_response_header_checksum);
-        connection->request->request_level_running_response_sum = NULL;
+        aws_checksum_destroy(request->request_level_running_response_sum);
+        aws_byte_buf_clean_up(&request->request_level_response_header_checksum);
+        request->request_level_running_response_sum = NULL;
     } else {
-        connection->request->did_validate = false;
+        request->did_validate = false;
     }
 }
 
@@ -946,12 +959,12 @@ static int s_s3_meta_request_incoming_headers(
     bool successful_response =
         s_s3_meta_request_error_code_from_response_status(request->send_data.response_status) == AWS_ERROR_SUCCESS;
 
-    if (successful_response && connection->request->meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT &&
-        connection->request->request_tag == AWS_S3_AUTO_RANGE_GET_REQUEST_TYPE_PART) {
-        s_get_part_response_headers_checksum_helper(connection, headers, headers_count);
+    if (successful_response && meta_request->checksum_config.validate_response_checksum &&
+        request->request_tag == AWS_S3_AUTO_RANGE_GET_REQUEST_TYPE_PART) {
+        s_get_part_response_headers_checksum_helper(connection, meta_request, headers, headers_count);
     }
 
-    /* Only record headers if an error has taken place, or if the reqest_desc has asked for them. */
+    /* Only record headers if an error has taken place, or if the request_desc has asked for them. */
     bool should_record_headers = !successful_response || request->record_response_headers;
 
     if (should_record_headers) {
@@ -998,8 +1011,8 @@ static int s_s3_meta_request_incoming_body(
         AWS_LOGF_TRACE(AWS_LS_S3_META_REQUEST, "response body: \n" PRInSTR "\n", AWS_BYTE_CURSOR_PRI(*data));
     }
 
-    if (connection->request->meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
-        s_get_part_response_body_checksum_helper(connection->request->request_level_running_response_sum, data);
+    if (meta_request->checksum_config.validate_response_checksum) {
+        s_get_part_response_body_checksum_helper(request->request_level_running_response_sum, data);
     }
 
     if (request->send_data.response_body.capacity == 0) {
@@ -1033,7 +1046,7 @@ static void s_s3_meta_request_stream_complete(struct aws_http_stream *stream, in
 
     struct aws_s3_connection *connection = user_data;
     AWS_PRECONDITION(connection);
-    if (connection->request->meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
+    if (connection->request->meta_request->checksum_config.validate_response_checksum) {
         s_get_response_part_finish_checksum_helper(connection, error_code);
     }
     s_s3_meta_request_send_request_finish(connection, stream, error_code);
@@ -1276,7 +1289,7 @@ static void s_s3_meta_request_body_streaming_task(struct aws_task *task, void *a
         if (aws_s3_meta_request_has_finish_result(meta_request)) {
             ++num_failed;
         } else {
-            if (error_code == AWS_ERROR_SUCCESS && meta_request->body_callback &&
+            if (body_buffer_byte_cursor.len > 0 && error_code == AWS_ERROR_SUCCESS && meta_request->body_callback &&
                 meta_request->body_callback(
                     meta_request, &body_buffer_byte_cursor, request->part_range_start, meta_request->user_data)) {
                 error_code = aws_last_error_or_unknown();
@@ -1444,13 +1457,9 @@ void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request
 
     aws_s3_meta_request_result_clean_up(meta_request, &finish_result);
 
-    /* The endpoint must be created by client here */
-    aws_s3_client_endpoint_release(meta_request->client, meta_request->endpoint);
+    aws_s3_endpoint_release(meta_request->endpoint);
     meta_request->endpoint = NULL;
 
-    aws_s3_client_release(meta_request->client);
-    meta_request->client = NULL;
-
     meta_request->io_event_loop = NULL;
 }
 
@@ -1511,3 +1520,22 @@ void aws_s3_meta_request_result_clean_up(
 
     AWS_ZERO_STRUCT(*result);
 }
+
+bool aws_s3_meta_request_checksum_config_has_algorithm(
+    struct aws_s3_meta_request *meta_request,
+    enum aws_s3_checksum_algorithm algorithm) {
+    AWS_PRECONDITION(meta_request);
+
+    switch (algorithm) {
+        case AWS_SCA_CRC32C:
+            return meta_request->checksum_config.response_checksum_algorithms.crc32c;
+        case AWS_SCA_CRC32:
+            return meta_request->checksum_config.response_checksum_algorithms.crc32;
+        case AWS_SCA_SHA1:
+            return meta_request->checksum_config.response_checksum_algorithms.sha1;
+        case AWS_SCA_SHA256:
+            return meta_request->checksum_config.response_checksum_algorithms.sha256;
+        default:
+            return false;
+    }
+}
