@@ -26,8 +26,8 @@ static void s_s3_client_acquire_http_connection_exceed_retries(
     AWS_ASSERT(callback);
     (void)conn_manager;
 
-    aws_raise_error(AWS_ERROR_UNKNOWN);
-    callback(NULL, AWS_ERROR_UNKNOWN, user_data);
+    aws_raise_error(AWS_ERROR_HTTP_UNKNOWN);
+    callback(NULL, AWS_ERROR_HTTP_UNKNOWN, user_data);
 }
 
 AWS_TEST_CASE(test_s3_client_exceed_retries, s_test_s3_client_exceed_retries)
@@ -36,7 +36,7 @@ static int s_test_s3_client_exceed_retries(struct aws_allocator *allocator, void
 
     struct aws_s3_tester tester;
     AWS_ZERO_STRUCT(tester);
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct aws_s3_client_config client_config;
     AWS_ZERO_STRUCT(client_config);
@@ -50,13 +50,13 @@ static int s_test_s3_client_exceed_retries(struct aws_allocator *allocator, void
     patched_client_vtable->acquire_http_connection = s_s3_client_acquire_http_connection_exceed_retries;
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
     /* Don't specify EXPECT SUCCESS flag for aws_s3_tester_send_get_object_meta_request to expect a failure. */
     ASSERT_SUCCESS(aws_s3_tester_send_get_object_meta_request(
         &tester, client, g_s3_path_get_object_test_1MB, 0, &meta_request_test_results));
 
-    ASSERT_TRUE(meta_request_test_results.finished_error_code == AWS_IO_MAX_RETRIES_EXCEEDED);
+    ASSERT_TRUE(meta_request_test_results.finished_error_code == AWS_ERROR_HTTP_UNKNOWN);
 
     aws_s3_meta_request_test_results_clean_up(&meta_request_test_results);
 
@@ -74,7 +74,7 @@ static void s_s3_client_acquire_http_connection_fail_first(
 
     struct aws_s3_connection *connection = user_data;
 
-    struct aws_s3_client *client = connection->request->meta_request->endpoint->user_data;
+    struct aws_s3_client *client = connection->request->meta_request->endpoint->client;
     AWS_ASSERT(client);
 
     struct aws_s3_tester *tester = client->shutdown_callback_user_data;
@@ -98,7 +98,7 @@ static int s_test_s3_client_acquire_connection_fail(struct aws_allocator *alloca
 
     struct aws_s3_tester tester;
     AWS_ZERO_STRUCT(tester);
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct aws_s3_client_config client_config = {.part_size = 64 * 1024};
 
@@ -177,7 +177,7 @@ static int s_test_s3_meta_request_fail_prepare_request(struct aws_allocator *all
     (void)ctx;
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct s3_fail_prepare_test_data test_data;
     AWS_ZERO_STRUCT(test_data);
@@ -247,7 +247,7 @@ static int s_test_s3_meta_request_sign_request_fail(struct aws_allocator *alloca
     (void)ctx;
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct aws_s3_client *client = NULL;
     struct aws_s3_tester_client_options client_options;
@@ -258,7 +258,7 @@ static int s_test_s3_meta_request_sign_request_fail(struct aws_allocator *alloca
     patched_client_vtable->meta_request_factory = s_s3_meta_request_factory_sign_request;
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
     struct aws_s3_tester_meta_request_options options = {
         .allocator = allocator,
@@ -363,7 +363,7 @@ static int s_test_s3_meta_request_send_request_finish_fail(struct aws_allocator
     (void)ctx;
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct aws_s3_client_config client_config = {
         .part_size = 64 * 1024,
@@ -429,7 +429,7 @@ static int s_test_s3_auto_range_put_missing_upload_id(struct aws_allocator *allo
     (void)ctx;
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct aws_s3_client *client = NULL;
     struct aws_s3_tester_client_options client_options;
@@ -440,7 +440,7 @@ static int s_test_s3_auto_range_put_missing_upload_id(struct aws_allocator *allo
     patched_client_vtable->meta_request_factory = s_meta_request_factory_patch_finished_request;
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
     struct aws_s3_tester_meta_request_options options = {
         .allocator = allocator,
