@@ -39,6 +39,12 @@ const struct aws_byte_cursor g_test_public_bucket_name =
 const struct aws_byte_cursor g_s3_path_get_object_test_1MB =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("/get_object_test_1MB.txt");
 const struct aws_byte_cursor g_s3_sse_header = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption");
+const struct aws_byte_cursor g_s3_sse_c_alg_header =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-algorithm");
+const struct aws_byte_cursor g_s3_sse_c_key_header =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-key");
+const struct aws_byte_cursor g_s3_sse_c_key_md5_header =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-key-md5");
 
 /* TODO populate these at the beginning of running tests with names that are unique to the test run. */
 const struct aws_byte_cursor g_pre_existing_object_1MB =
@@ -98,6 +104,7 @@ static int s_s3_test_meta_request_body_callback(
 
     struct aws_s3_meta_request_test_results *meta_request_test_results = user_data;
     meta_request_test_results->received_body_size += body->len;
+    aws_atomic_fetch_add(&meta_request_test_results->received_body_size_delta, body->len);
 
     AWS_LOGF_DEBUG(
         AWS_LS_S3_GENERAL,
@@ -331,6 +338,15 @@ int aws_s3_tester_bind_meta_request(
     return AWS_OP_SUCCESS;
 }
 
+void aws_s3_meta_request_test_results_init(
+    struct aws_s3_meta_request_test_results *test_meta_request,
+    struct aws_allocator *allocator) {
+
+    (void)allocator;
+    AWS_ZERO_STRUCT(*test_meta_request);
+    aws_atomic_init_int(&test_meta_request->received_body_size_delta, 0);
+}
+
 void aws_s3_meta_request_test_results_clean_up(struct aws_s3_meta_request_test_results *test_meta_request) {
     if (test_meta_request == NULL) {
         return;
@@ -589,9 +605,9 @@ static void s_s3_client_process_work_empty(struct aws_s3_client *client) {
     (void)client;
 }
 
-static void s_s3_client_endpoint_shutdown_callback_empty(void *user_data) {
-    AWS_PRECONDITION(user_data);
-    (void)user_data;
+static void s_s3_client_endpoint_shutdown_callback_empty(struct aws_s3_client *client) {
+    AWS_PRECONDITION(client);
+    (void)client;
 }
 
 static void s_s3_client_finish_destroy_empty(struct aws_s3_client *client) {
@@ -742,17 +758,29 @@ struct aws_s3_empty_meta_request {
     struct aws_s3_meta_request base;
 };
 
-static void s_s3_mock_endpoint_zero_ref(void *user_data) {
-    struct aws_s3_endpoint *endpoint = user_data;
-
-    aws_string_destroy(endpoint->host_name);
-    aws_mem_release(endpoint->allocator, endpoint);
+static void s_s3_mock_endpoint_acquire(struct aws_s3_endpoint *endpoint, bool already_holding_lock) {
+    (void)already_holding_lock;
+    ++endpoint->client_synced_data.ref_count;
 }
 
+static void s_s3_mock_endpoint_release(struct aws_s3_endpoint *endpoint) {
+    if (--endpoint->client_synced_data.ref_count == 0) {
+        aws_string_destroy(endpoint->host_name);
+        aws_mem_release(endpoint->allocator, endpoint);
+    }
+}
+
+static struct aws_s3_endpoint_system_vtable s_s3_mock_endpoint_vtable = {
+    .acquire = s_s3_mock_endpoint_acquire,
+    .release = s_s3_mock_endpoint_release,
+};
+
 struct aws_s3_endpoint *aws_s3_tester_mock_endpoint_new(struct aws_s3_tester *tester) {
+    aws_s3_endpoint_set_system_vtable(&s_s3_mock_endpoint_vtable);
+
     struct aws_s3_endpoint *endpoint = aws_mem_calloc(tester->allocator, 1, sizeof(struct aws_s3_endpoint));
     endpoint->allocator = tester->allocator;
-    aws_ref_count_init(&endpoint->ref_count, endpoint, s_s3_mock_endpoint_zero_ref);
+    endpoint->client_synced_data.ref_count = 1;
 
     struct aws_byte_cursor empty_cursor = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("");
     endpoint->host_name = aws_string_new_from_cursor(tester->allocator, &empty_cursor);
@@ -779,8 +807,6 @@ struct aws_s3_meta_request *aws_s3_tester_mock_meta_request_new(struct aws_s3_te
         NULL,
         0,
         false,
-        AWS_SCA_NONE,
-        false,
         &options,
         empty_meta_request,
         &s_s3_mock_meta_request_vtable,
@@ -1023,6 +1049,26 @@ struct aws_http_message *aws_s3_test_put_object_request_new(
         }
     }
 
+    if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_C_AES256) {
+        struct aws_http_header sse_c_alg_header = {
+            .name = g_s3_sse_c_alg_header,
+            .value = aws_byte_cursor_from_c_str("AES256"),
+        };
+        struct aws_http_header sse_c_key_header = {
+            .name = g_s3_sse_c_key_header,
+            .value = aws_byte_cursor_from_c_str("MDAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDAwMDE="),
+        };
+        struct aws_http_header sse_c_key_md5_header = {
+            .name = g_s3_sse_c_key_md5_header,
+            .value = aws_byte_cursor_from_c_str("5inxltxhJQnqmmHfVSQjoA=="),
+        };
+        if (aws_http_message_add_header(message, sse_c_alg_header) ||
+            aws_http_message_add_header(message, sse_c_key_header) ||
+            aws_http_message_add_header(message, sse_c_key_md5_header)) {
+            goto error_clean_up_message;
+        }
+    }
+
     if (flags & AWS_S3_TESTER_SEND_META_REQUEST_PUT_ACL) {
         if (aws_http_message_add_header(message, acl_public_read_header)) {
             goto error_clean_up_message;
@@ -1147,11 +1193,17 @@ int aws_s3_tester_send_meta_request_with_options(
         aws_s3_client_acquire(client);
     }
 
+    struct aws_s3_checksum_config checksum_config = {
+        .checksum_algorithm = options->checksum_algorithm,
+        .validate_response_checksum = options->validate_get_response_checksum,
+        .location = options->checksum_algorithm == AWS_SCA_NONE ? AWS_SCL_NONE : AWS_SCL_TRAILER,
+        .validate_checksum_algorithms = options->validate_checksum_algorithms,
+    };
+
     struct aws_s3_meta_request_options meta_request_options = {
         .type = options->meta_request_type,
         .message = options->message,
-        .checksum_algorithm = options->checksum_algorithm,
-        .validate_get_response_checksum = options->validate_get_response_checksum,
+        .checksum_config = &checksum_config,
     };
 
     if (options->signing_config) {
@@ -1199,6 +1251,8 @@ int aws_s3_tester_send_meta_request_with_options(
             (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_DEFAULT &&
              options->default_type_options.mode == AWS_S3_TESTER_DEFAULT_TYPE_MODE_PUT)) {
 
+            meta_request_options.checksum_config = &checksum_config;
+
             uint32_t object_size_mb = options->put_options.object_size_mb;
             size_t object_size_bytes = (size_t)object_size_mb * 1024ULL * 1024ULL;
 
@@ -1249,6 +1303,13 @@ int aws_s3_tester_send_meta_request_with_options(
                             "/put_object_test_aes256_%uMB.txt",
                             object_size_mb);
                         break;
+                    case AWS_S3_TESTER_SSE_C_AES256:
+                        snprintf(
+                            object_path_sprintf_buffer,
+                            sizeof(object_path_sprintf_buffer),
+                            "/put_object_test_c_aes256_%uMB.txt",
+                            object_size_mb);
+                        break;
 
                     default:
                         break;
@@ -1298,7 +1359,7 @@ int aws_s3_tester_send_meta_request_with_options(
     }
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
     if (out_results == NULL) {
         out_results = &meta_request_test_results;
@@ -1308,7 +1369,7 @@ int aws_s3_tester_send_meta_request_with_options(
     out_results->body_callback = options->body_callback;
     out_results->finish_callback = options->finish_callback;
 
-    out_results->algorithm = options->checksum_algorithm;
+    out_results->algorithm = options->expected_validate_checksum_alg;
 
     ASSERT_SUCCESS(aws_s3_tester_bind_meta_request(tester, &meta_request_options, out_results));
 
@@ -1445,7 +1506,7 @@ int aws_s3_tester_send_get_object_meta_request(
 
     /* Trigger accelerating of our Get Object request. */
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, tester->allocator);
 
     if (out_results == NULL) {
         out_results = &meta_request_test_results;
@@ -1510,6 +1571,12 @@ int aws_s3_tester_validate_get_object_results(
         ASSERT_TRUE(aws_byte_cursor_eq_c_str(&sse_byte_cursor, "AES256"));
     }
 
+    if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_C_AES256) {
+        ASSERT_SUCCESS(
+            aws_http_headers_get(meta_request_test_results->response_headers, g_s3_sse_c_alg_header, &sse_byte_cursor));
+        ASSERT_TRUE(aws_byte_cursor_eq_c_str(&sse_byte_cursor, "AES256"));
+    }
+
     struct aws_string *content_length_str = aws_string_new_from_cursor(tester->allocator, &content_length_cursor);
 
     char *content_length_str_end = NULL;
@@ -1553,7 +1620,7 @@ int aws_s3_tester_send_put_object_meta_request(
 
     if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_KMS) {
         snprintf(object_path_buffer, sizeof(object_path_buffer), "/get_object_test_kms_%uMB.txt", file_size_mb);
-    } else if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_KMS) {
+    } else if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_AES256) {
         snprintf(object_path_buffer, sizeof(object_path_buffer), "/get_object_test_aes256_%uMB.txt", file_size_mb);
     } else if (flags & AWS_S3_TESTER_SEND_META_REQUEST_PUT_ACL) {
         snprintf(
@@ -1588,7 +1655,7 @@ int aws_s3_tester_send_put_object_meta_request(
     options.message = message;
 
     struct aws_s3_meta_request_test_results meta_request_test_results;
-    AWS_ZERO_STRUCT(meta_request_test_results);
+    aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
     if (out_results == NULL) {
         out_results = &meta_request_test_results;
@@ -1645,6 +1712,12 @@ int aws_s3_tester_validate_put_object_results(
     }
     ASSERT_TRUE(etag_byte_cursor.len > 0);
 
+    if (flags & AWS_S3_TESTER_SEND_META_REQUEST_SSE_C_AES256) {
+        ASSERT_SUCCESS(
+            aws_http_headers_get(meta_request_test_results->response_headers, g_s3_sse_c_alg_header, &sse_byte_cursor));
+        ASSERT_TRUE(aws_byte_cursor_eq_c_str(&sse_byte_cursor, "AES256"));
+    }
+
     struct aws_byte_cursor quote_entity = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("&quot;");
 
     if (etag_byte_cursor.len >= quote_entity.len) {
