@@ -220,7 +220,7 @@ beeu_mod_inverse_vartime:
     //    sp  <------------------- 112 bytes ----------------> old sp
     //   x29 (FP)
     //
-    .inst   0xd503233f      // paciasp
+    AARCH64_SIGN_LINK_REGISTER
     stp     x29,x30,[sp,#-112]!
     add     x29,sp,#0
     stp     x19,x20,[sp,#16]
@@ -428,7 +428,7 @@ beeu_mod_inverse_vartime:
     ldp     x27,x28,[sp,#80]
     ldp     x29,x30,[sp],#112
 
-    .inst   0xd50323bf      // autiasp
+    AARCH64_VALIDATE_LINK_REGISTER
     ret
 .size beeu_mod_inverse_vartime,.-beeu_mod_inverse_vartime
 ___
