@@ -27,6 +27,7 @@
 #endif
 
 #include "api/s2n.h"
+#include "api/unstable/renegotiate.h"
 #include "common.h"
 #include "error/s2n_errno.h"
 
@@ -34,6 +35,8 @@
 
 #define OPT_TICKET_IN 1000
 #define OPT_TICKET_OUT 1001
+#define OPT_SEND_FILE 1002
+#define OPT_RENEG 1003
 
 void usage()
 {
@@ -51,6 +54,8 @@ void usage()
     fprintf(stderr, "    Enter libcrypto's FIPS mode. The linked version of OpenSSL must be built with the FIPS module.\n");
     fprintf(stderr, "  -e,--echo\n");
     fprintf(stderr, "    Listen to stdin after TLS Connection is established and echo it to the Server\n");
+    fprintf(stderr, "  --send-file [file path]\n");
+    fprintf(stderr, "    Sends the contents of the provided file to the server after connecting.\n");
     fprintf(stderr, "  -h,--help\n");
     fprintf(stderr, "    Display this message and quit.\n");
     fprintf(stderr, "  -n [server name]\n");
@@ -94,13 +99,14 @@ void usage()
                     "    Note that the maximum number of permitted psks is 10, the psk-secret is hex-encoded, and whitespace is not allowed before or after the commas.\n"
                     "    Ex: --psk psk_id,psk_secret,SHA256 --psk shared_id,shared_secret,SHA384.\n");
     fprintf(stderr, "  -E ,--early-data <file path>\n");
-    fprintf(stderr, "    Sends data in file path as early data to the server. Early data will only be sent if s2nc receives a session ticket and resumes a session.");
+    fprintf(stderr, "    Sends data in file path as early data to the server. Early data will only be sent if s2nc receives a session ticket and resumes a session.\n");
+    fprintf(stderr, "  --renegotiation [accept|reject]\n"
+                    "    accept: Accept all server requests for a new handshake\n"
+                    "    reject: Reject all server requests for a new handshake\n");
     fprintf(stderr, "\n");
     exit(1);
 }
 
-
-
 size_t session_state_length = 0;
 uint8_t *session_state = NULL;
 static int test_session_ticket_cb(struct s2n_connection *conn, void *ctx, struct s2n_session_ticket *ticket)
@@ -122,6 +128,25 @@ static int test_session_ticket_cb(struct s2n_connection *conn, void *ctx, struct
     return S2N_SUCCESS;
 }
 
+struct reneg_req_ctx {
+    bool do_renegotiate;
+    s2n_renegotiate_response response;
+};
+
+static int reneg_req_cb(struct s2n_connection *conn, void *context, s2n_renegotiate_response *response)
+{
+    GUARD_EXIT_NULL(conn);
+    GUARD_EXIT_NULL(context);
+    GUARD_EXIT_NULL(response);
+    struct reneg_req_ctx *reneg_ctx = (struct reneg_req_ctx *) context;
+
+    *response = reneg_ctx->response;
+    if (*response == S2N_RENEGOTIATE_ACCEPT) {
+        reneg_ctx->do_renegotiate = true;
+    }
+    return S2N_SUCCESS;
+}
+
 static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs, s2n_status_request_type type,
     struct verify_data *unsafe_verify_data, const char *host, const char *alpn_protocols, uint16_t mfl_value) {
 
@@ -258,7 +283,8 @@ int main(int argc, char *const *argv)
     const char *host = NULL;
     struct verify_data unsafe_verify_data;
     const char *port = "443";
-    int echo_input = 0;
+    bool echo_input = false;
+    const char *send_file = NULL;
     int use_corked_io = 0;
     uint8_t non_blocking = 0;
     const char *key_log_path = NULL;
@@ -266,12 +292,15 @@ int main(int argc, char *const *argv)
     char *psk_optarg_list[S2N_MAX_PSK_LIST_LENGTH];
     size_t psk_list_len = 0;
     char *early_data = NULL;
+    bool setup_reneg_cb = false;
+    struct reneg_req_ctx reneg_ctx = { 0 };
 
     static struct option long_options[] = {
         {"alpn", required_argument, 0, 'a'},
         {"ciphers", required_argument, 0, 'c'},
         {"enter-fips-mode", no_argument, NULL, 'F'},
         {"echo", no_argument, 0, 'e'},
+        {"send-file", required_argument, 0, OPT_SEND_FILE},
         {"help", no_argument, 0, 'h'},
         {"name", required_argument, 0, 'n'},
         {"status", no_argument, 0, 's'},
@@ -293,6 +322,7 @@ int main(int argc, char *const *argv)
         {"key-log", required_argument, 0, 'L'},
         {"psk", required_argument, 0, 'P'},
         {"early-data", required_argument, 0, 'E'},
+        {"renegotiation", required_argument, 0, OPT_RENEG},
         { 0 },
     };
 
@@ -316,7 +346,10 @@ int main(int argc, char *const *argv)
             fips_mode = 1;
             break;
         case 'e':
-            echo_input = 1;
+            echo_input = true;
+            break;
+        case OPT_SEND_FILE:
+            send_file = load_file_to_cstring(optarg);
             break;
         case 'h':
             usage();
@@ -389,6 +422,17 @@ int main(int argc, char *const *argv)
             early_data = load_file_to_cstring(optarg);
             GUARD_EXIT_NULL(early_data);
             break;
+        case OPT_RENEG:
+            setup_reneg_cb = true;
+            if (strcmp(optarg, "accept") == 0) {
+                reneg_ctx.response = S2N_RENEGOTIATE_ACCEPT;
+            } else if (strcmp(optarg, "reject") == 0) {
+                reneg_ctx.response = S2N_RENEGOTIATE_REJECT;
+            } else {
+                fprintf(stderr, "Unrecognized option: %s\n", optarg);
+                exit(1);
+            }
+            break;
         case '?':
         default:
             usage();
@@ -519,6 +563,11 @@ int main(int argc, char *const *argv)
             );
         }
 
+        if (setup_reneg_cb) {
+            GUARD_EXIT(s2n_config_set_renegotiate_request_cb(config, reneg_req_cb, &reneg_ctx),
+                    "Error setting renegotiation request callback");
+        }
+
         struct s2n_connection *conn = s2n_connection_new(S2N_CLIENT);
 
         if (conn == NULL) {
@@ -607,11 +656,25 @@ int main(int argc, char *const *argv)
 
         GUARD_EXIT(s2n_connection_free_handshake(conn), "Error freeing handshake memory after negotiation");
 
-        if (echo_input == 1) {
-            bool stop_echo = false;
+        if (send_file != NULL) {
+            printf("Sending file contents:\n%s\n", send_file);
+
+            unsigned long send_file_len = strlen(send_file);
+            s2n_blocked_status blocked = S2N_NOT_BLOCKED;
+            send_data(conn, sockfd, send_file, send_file_len, &blocked);
+        }
+
+        while (echo_input) {
             fflush(stdout);
             fflush(stderr);
-            echo(conn, sockfd, &stop_echo);
+            echo(conn, sockfd, &reneg_ctx.do_renegotiate);
+
+            if (!reneg_ctx.do_renegotiate) {
+                break;
+            }
+
+            reneg_ctx.do_renegotiate = false;
+            GUARD_EXIT(renegotiate(conn, sockfd), "Renegotiation failed");
         }
 
         /* The following call can block on receiving a close_notify if we initiate the shutdown or if the */
