@@ -13,24 +13,43 @@
  * permissions and limitations under the License.
  */
 
-#include "crypto/s2n_rsa.h"
-
 #include <openssl/evp.h>
 #include <openssl/rsa.h>
 #include <stdint.h>
 
-#include "crypto/s2n_drbg.h"
-#include "crypto/s2n_hash.h"
-#include "crypto/s2n_pkey.h"
-#include "crypto/s2n_rsa_signing.h"
 #include "error/s2n_errno.h"
 #include "stuffer/s2n_stuffer.h"
+
+#include "crypto/s2n_drbg.h"
+#include "crypto/s2n_evp_signing.h"
+#include "crypto/s2n_hash.h"
+#include "crypto/s2n_pkey.h"
+#include "crypto/s2n_rsa.h"
+#include "crypto/s2n_rsa_signing.h"
+
 #include "utils/s2n_blob.h"
+#include "utils/s2n_compiler.h"
 #include "utils/s2n_random.h"
 #include "utils/s2n_result.h"
 #include "utils/s2n_safety.h"
 
-static S2N_RESULT s2n_rsa_modulus_check(RSA *rsa)
+RSA *s2n_unsafe_rsa_get_non_const(const struct s2n_rsa_key *rsa_key) {
+    PTR_ENSURE_REF(rsa_key);
+
+    /* pragma gcc diagnostic was added in gcc 4.6 */
+#if defined(__clang__) || S2N_GCC_VERSION_AT_LEAST(4,6,0)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+    RSA *out_rsa_key = (RSA *) rsa_key->rsa;
+#if defined(__clang__) || S2N_GCC_VERSION_AT_LEAST(4,6,0)
+#pragma GCC diagnostic pop
+#endif
+
+    return out_rsa_key;
+}
+
+static S2N_RESULT s2n_rsa_modulus_check(const RSA *rsa)
 {
 /* RSA was made opaque starting in Openssl 1.1.0 */
 #if S2N_OPENSSL_VERSION_AT_LEAST(1, 1, 0) && !defined(LIBRESSL_VERSION_NUMBER)
@@ -44,13 +63,14 @@ static S2N_RESULT s2n_rsa_modulus_check(RSA *rsa)
     return S2N_RESULT_OK;
 }
 
-static S2N_RESULT s2n_rsa_encrypted_size(const struct s2n_pkey *key, uint32_t *size_out)
+static S2N_RESULT s2n_rsa_encrypted_size(const struct s2n_pkey *pkey, uint32_t *size_out)
 {
-    RESULT_ENSURE_REF(key);
+    RESULT_ENSURE_REF(pkey);
     RESULT_ENSURE_REF(size_out);
 
-    const struct s2n_rsa_key *rsa_key = &key->key.rsa_key;
+    const struct s2n_rsa_key *rsa_key = &pkey->key.rsa_key;
     RESULT_ENSURE_REF(rsa_key->rsa);
+
     RESULT_GUARD(s2n_rsa_modulus_check(rsa_key->rsa));
 
     const int size = RSA_size(rsa_key->rsa);
@@ -96,9 +116,11 @@ static int s2n_rsa_encrypt(const struct s2n_pkey *pub, struct s2n_blob *in, stru
     POSIX_GUARD_RESULT(s2n_rsa_encrypted_size(pub, &size));
     S2N_ERROR_IF(out->size < size, S2N_ERR_NOMEM);
 
-    const s2n_rsa_public_key *key = &pub->key.rsa_key;
-    int r = RSA_public_encrypt(in->size, ( unsigned char * )in->data, ( unsigned char * )out->data, key->rsa,
-                               RSA_PKCS1_PADDING);
+    const s2n_rsa_public_key *pub_key = &pub->key.rsa_key;
+
+    /* Safety: RSA_public_encrypt does not mutate the key */
+    int r = RSA_public_encrypt(in->size, ( unsigned char * )in->data, ( unsigned char * )out->data,
+            s2n_unsafe_rsa_get_non_const(pub_key), RSA_PKCS1_PADDING);
     S2N_ERROR_IF(r != out->size, S2N_ERR_SIZE_MISMATCH);
 
     return 0;
@@ -116,8 +138,11 @@ static int s2n_rsa_decrypt(const struct s2n_pkey *priv, struct s2n_blob *in, str
 
     POSIX_GUARD_RESULT(s2n_get_public_random_data(out));
 
-    const s2n_rsa_private_key *key = &priv->key.rsa_key;
-    int r = RSA_private_decrypt(in->size, ( unsigned char * )in->data, intermediate, key->rsa, RSA_NO_PADDING);
+    const s2n_rsa_private_key *priv_key = &priv->key.rsa_key;
+
+    /* Safety: RSA_private_decrypt does not mutate the key */
+    int r = RSA_private_decrypt(in->size, ( unsigned char * )in->data, intermediate,
+            s2n_unsafe_rsa_get_non_const(priv_key), RSA_NO_PADDING);
     S2N_ERROR_IF(r != expected_size, S2N_ERR_SIZE_MISMATCH);
 
     s2n_constant_time_pkcs1_unpad_or_dont(out->data, intermediate, r, out->size);
@@ -149,13 +174,17 @@ static int s2n_rsa_keys_match(const struct s2n_pkey *pub, const struct s2n_pkey
 
 static int s2n_rsa_key_free(struct s2n_pkey *pkey)
 {
+    POSIX_ENSURE_REF(pkey);
     struct s2n_rsa_key *rsa_key = &pkey->key.rsa_key;
-    if (rsa_key->rsa == NULL) { return 0; }
+    if (rsa_key->rsa == NULL) {
+        return S2N_SUCCESS;
+    }
 
-    RSA_free(rsa_key->rsa);
+    /* Safety: freeing the key owned by this object */
+    RSA_free(s2n_unsafe_rsa_get_non_const(rsa_key));
     rsa_key->rsa = NULL;
 
-    return 0;
+    return S2N_SUCCESS;
 }
 
 static int s2n_rsa_check_key_exists(const struct s2n_pkey *pkey)
@@ -167,7 +196,7 @@ static int s2n_rsa_check_key_exists(const struct s2n_pkey *pkey)
 
 int s2n_evp_pkey_to_rsa_public_key(s2n_rsa_public_key *rsa_key, EVP_PKEY *evp_public_key)
 {
-    RSA *rsa = EVP_PKEY_get1_RSA(evp_public_key);
+    const RSA *rsa = EVP_PKEY_get1_RSA(evp_public_key);
     S2N_ERROR_IF(rsa == NULL, S2N_ERR_DECODE_CERTIFICATE);
 
     rsa_key->rsa = rsa;
@@ -176,7 +205,7 @@ int s2n_evp_pkey_to_rsa_public_key(s2n_rsa_public_key *rsa_key, EVP_PKEY *evp_pu
 
 int s2n_evp_pkey_to_rsa_private_key(s2n_rsa_private_key *rsa_key, EVP_PKEY *evp_private_key)
 {
-    RSA *rsa = EVP_PKEY_get1_RSA(evp_private_key);
+    const RSA *rsa = EVP_PKEY_get1_RSA(evp_private_key);
     S2N_ERROR_IF(rsa == NULL, S2N_ERR_DECODE_PRIVATE_KEY);
 
     rsa_key->rsa = rsa;
@@ -193,6 +222,7 @@ int s2n_rsa_pkey_init(struct s2n_pkey *pkey)
     pkey->match     = &s2n_rsa_keys_match;
     pkey->free      = &s2n_rsa_key_free;
     pkey->check_key = &s2n_rsa_check_key_exists;
+    POSIX_GUARD_RESULT(s2n_evp_signing_set_pkey_overrides(pkey));
     return 0;
 }
 
