@@ -59,6 +59,7 @@ typedef enum {
     S2N_ERR_IO_BLOCKED = S2N_ERR_T_BLOCKED_START,
     S2N_ERR_ASYNC_BLOCKED,
     S2N_ERR_EARLY_DATA_BLOCKED,
+    S2N_ERR_APP_DATA_BLOCKED,
     S2N_ERR_T_BLOCKED_END,
 
     /* S2N_ERR_T_ALERT */
@@ -128,6 +129,7 @@ typedef enum {
     S2N_ERR_DUPLICATE_EXTENSION,
     S2N_ERR_MAX_EARLY_DATA_SIZE,
     S2N_ERR_EARLY_DATA_TRIAL_DECRYPT,
+    S2N_ERR_NO_RENEGOTIATION,
     S2N_ERR_T_PROTO_END,
 
     /* S2N_ERR_T_INTERNAL */
@@ -215,6 +217,7 @@ typedef enum {
     S2N_ERR_RETRIEVE_FORK_GENERATION_NUMBER,
     S2N_ERR_LIBCRYPTO_VERSION_NUMBER_MISMATCH,
     S2N_ERR_LIBCRYPTO_VERSION_NAME_MISMATCH,
+    S2N_ERR_OSSL_PROVIDER,
     S2N_ERR_T_INTERNAL_END,
 
     /* S2N_ERR_T_USAGE */
