@@ -49,6 +49,7 @@
 #include "tls/extensions/s2n_server_supported_versions.h"
 #include "tls/extensions/s2n_server_key_share.h"
 #include "tls/extensions/s2n_server_psk.h"
+#include "tls/extensions/s2n_npn.h"
 
 static const s2n_extension_type *const client_hello_extensions[] = {
         &s2n_client_supported_versions_extension,
@@ -74,6 +75,7 @@ static const s2n_extension_type *const client_hello_extensions[] = {
         &s2n_psk_key_exchange_modes_extension,
         &s2n_client_early_data_indication_extension,
         &s2n_client_ems_extension,
+        &s2n_client_npn_extension,
         &s2n_client_psk_extension /* MUST be last */
 };
 
@@ -88,6 +90,7 @@ static const s2n_extension_type *const tls12_server_hello_extensions[] = {
         &s2n_server_max_fragment_length_extension,
         &s2n_server_session_ticket_extension,
         &s2n_server_ems_extension,
+        &s2n_server_npn_extension,
 };
 
 /**
@@ -121,6 +124,10 @@ static const s2n_extension_type *const encrypted_extensions[] = {
         &s2n_server_early_data_indication_extension,
 };
 
+static const s2n_extension_type *const tls12_encrypted_extensions[] = {
+        &s2n_npn_encrypted_extension,
+};
+
 static const s2n_extension_type *const cert_req_extensions[] = {
         &s2n_server_signature_algorithms_extension,
 };
@@ -145,6 +152,7 @@ static s2n_extension_type_list extension_lists[] = {
         [S2N_EXTENSION_LIST_CERT_REQ] = S2N_EXTENSION_LIST(cert_req_extensions),
         [S2N_EXTENSION_LIST_CERTIFICATE] = S2N_EXTENSION_LIST(certificate_extensions),
         [S2N_EXTENSION_LIST_NST] = S2N_EXTENSION_LIST(nst_extensions),
+        [S2N_EXTENSION_LIST_ENCRYPTED_EXTENSIONS_TLS12] = S2N_EXTENSION_LIST(tls12_encrypted_extensions),
         [S2N_EXTENSION_LIST_EMPTY] = { .extension_types = NULL, .count = 0 },
 };
 
