@@ -22,7 +22,7 @@
 
 #include "tls/extensions/s2n_server_alpn.h"
 
-static bool s2n_alpn_should_send(struct s2n_connection *conn);
+bool s2n_server_alpn_should_send(struct s2n_connection *conn);
 static int s2n_alpn_send(struct s2n_connection *conn, struct s2n_stuffer *out);
 static int s2n_alpn_recv(struct s2n_connection *conn, struct s2n_stuffer *extension);
 
@@ -31,11 +31,11 @@ const s2n_extension_type s2n_server_alpn_extension = {
     .is_response = true,
     .send = s2n_alpn_send,
     .recv = s2n_alpn_recv,
-    .should_send = s2n_alpn_should_send,
+    .should_send = s2n_server_alpn_should_send,
     .if_missing = s2n_extension_noop_if_missing,
 };
 
-static bool s2n_alpn_should_send(struct s2n_connection *conn)
+bool s2n_server_alpn_should_send(struct s2n_connection *conn)
 {
     return conn && strlen(conn->application_protocol) > 0;
 }
