@@ -33,22 +33,6 @@ uint8_t hello_retry_req_random[S2N_TLS_RANDOM_DATA_LEN] = {
     0xC2, 0xA2, 0x11, 0x16, 0x7A, 0xBB, 0x8C, 0x5E, 0x07, 0x9E, 0x09, 0xE2, 0xC8, 0xA8, 0x33, 0x9C
 };
 
-static int s2n_conn_reset_retry_values(struct s2n_connection *conn)
-{
-    POSIX_ENSURE_REF(conn);
-
-    /* Reset handshake values */
-    conn->handshake.client_hello_received = 0;
-
-    /* Reset client hello state */
-    POSIX_GUARD(s2n_stuffer_wipe(&conn->client_hello.raw_message));
-    POSIX_GUARD(s2n_stuffer_resize(&conn->client_hello.raw_message, 0));
-    POSIX_GUARD(s2n_client_hello_free(&conn->client_hello));
-    POSIX_GUARD(s2n_stuffer_growable_alloc(&conn->client_hello.raw_message, 0));
-
-    return 0;
-}
-
 int s2n_server_hello_retry_send(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
@@ -62,7 +46,11 @@ int s2n_server_hello_retry_send(struct s2n_connection *conn)
 
     /* Update transcript */
     POSIX_GUARD(s2n_server_hello_retry_recreate_transcript(conn));
-    POSIX_GUARD(s2n_conn_reset_retry_values(conn));
+
+    /* Reset handshake values */
+    conn->handshake.client_hello_received = 0;
+    conn->client_hello.parsed = 0;
+    POSIX_CHECKED_MEMSET((uint8_t*) conn->extension_requests_received, 0, sizeof(s2n_extension_bitfield));
 
     return 0;
 }
@@ -80,20 +68,33 @@ int s2n_server_hello_retry_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_connection_get_kem_preferences(conn, &kem_pref));
     POSIX_ENSURE_REF(kem_pref);
 
-    /* Upon receipt of the HelloRetryRequest, the client MUST verify that:
-     * (1) the selected_group field corresponds to a group
-     * which was provided in the "supported_groups" extension in the
-     * original ClientHello and
-     * (2) the selected_group field does not correspond to a group which was provided
-     * in the "key_share" extension in the original ClientHello.
-     * If either of these checks fails, then the client MUST abort the handshake. */
-
     const struct s2n_ecc_named_curve *named_curve = conn->kex_params.server_ecc_evp_params.negotiated_curve;
     const struct s2n_kem_group *kem_group = conn->kex_params.server_kem_group_params.kem_group;
 
     /* Boolean XOR check: exactly one of {named_curve, kem_group} should be non-null. */
     POSIX_ENSURE( (named_curve != NULL) != (kem_group != NULL), S2N_ERR_INVALID_HELLO_RETRY);
 
+    /**
+     *= https://tools.ietf.org/rfc/rfc8446#4.2.8
+     *# Upon receipt of this extension in a HelloRetryRequest, the client
+     *# MUST verify that (1) the selected_group field corresponds to a group
+     *# which was provided in the "supported_groups" extension in the
+     *# original ClientHello
+     **/
+    bool selected_group_in_supported_groups = false;
+    if (named_curve != NULL && s2n_ecc_preferences_includes_curve(ecc_pref, named_curve->iana_id)) {
+        selected_group_in_supported_groups = true;
+    }
+    if (kem_group != NULL && s2n_kem_preferences_includes_tls13_kem_group(kem_pref, kem_group->iana_id)) {
+        selected_group_in_supported_groups = true;
+    }
+
+    /**
+     *= https://tools.ietf.org/rfc/rfc8446#4.2.8
+     *# and (2) the selected_group field does not
+     *# correspond to a group which was provided in the "key_share" extension
+     *# in the original ClientHello.
+     **/
     bool new_key_share_requested = false;
     if (named_curve != NULL) {
         new_key_share_requested = (named_curve != conn->kex_params.client_ecc_evp_params.negotiated_curve);
@@ -105,17 +106,25 @@ int s2n_server_hello_retry_recv(struct s2n_connection *conn)
         new_key_share_requested = (kem_group != conn->kex_params.client_kem_group_params.kem_group);
     }
 
-    /*
+    /**
+     *= https://tools.ietf.org/rfc/rfc8446#4.2.8
+     *# If either of these checks fails, then
+     *# the client MUST abort the handshake with an "illegal_parameter"
+     *# alert.
+     * 
      *= https://tools.ietf.org/rfc/rfc8446#section-4.1.4
      *# Clients MUST abort the handshake with an
      *# "illegal_parameter" alert if the HelloRetryRequest would not result
      *# in any change in the ClientHello.
-     */
-    POSIX_ENSURE((conn->early_data_state == S2N_EARLY_DATA_REJECTED) || new_key_share_requested,
-            S2N_ERR_INVALID_HELLO_RETRY);
+     **/
+    POSIX_ENSURE(new_key_share_requested, S2N_ERR_INVALID_HELLO_RETRY);
+    POSIX_ENSURE(selected_group_in_supported_groups, S2N_ERR_INVALID_HELLO_RETRY);
 
     /* Update transcript hash */
     POSIX_GUARD(s2n_server_hello_retry_recreate_transcript(conn));
 
+    /* Reset handshake values */
+    POSIX_CHECKED_MEMSET((uint8_t*) conn->extension_requests_sent, 0, sizeof(s2n_extension_bitfield));
+
     return S2N_SUCCESS;
 }
